from typing import List, Literal

from devopness.models import (
    Action,
    ActionPipelineCreatePlain,
    Service,
    ServiceRelation,
    ServiceType,
)

from ..devopness_api import devopness, ensure_authenticated
from ..response import MCPResponse


class ServiceService:
    @staticmethod
    async def tool_create_service(
        operation: Literal[
            "create",
            "list_available_service_types",
            "list_available_service_type_versions",
        ],
        environment_id: int,
        service_type: ServiceType | None = None,
        service_version: str | None = None,
    ) -> MCPResponse[Service] | None:
        """
        Rules:
        1. DO NOT execute this tool without first confirming with the user which
          environment ID to use.

        2. DO NOT execute this tool without first confirming with the user which
          service type to use.

        3. DO NOT execute this tool without first confirming with the user which
          service version to use.

        Usage:
        1. To create a service:
            - Call this function with `operation='create'`, providing a valid
              `environment_id`, `service_type`, and `service_version`.
            - Example: devopness_create_service(
                operation='create',
                environment_id=123,
                service_type=ServiceType.DOCKER,
                service_version='20.10',
            )

        2. To list available service types:
            - Call this function with `operation='list_available_service_types'`,
              providing a valid `environment_id`.
            - Example: devopness_create_service(
                operation='list_available_service_types',
                environment_id=123,
            )

        3. To list available service type versions:
            - Call this function with `operation='list_available_service_type_versions'`
              providing a valid `environment_id` and `service_type`.
            - Example: devopness_create_service(
                operation='list_available_service_type_versions',
                environment_id=123,
                service_type=ServiceType.DOCKER,
            )
        """
        await ensure_authenticated()

        match operation:
            case "create":
                if not service_type or not service_version:
                    return MCPResponse.error(
                        [
                            "Both service type and version are required to create"
                            " a service. "
                            "Please ask the user to provide these details."
                        ]
                    )

                response = await devopness.services.add_environment_service(
                    environment_id,
                    {
                        "type": service_type,
                        "version": service_version,
                    },
                )

                return MCPResponse.ok(
                    response.data,
                    [
                        "Service has been successfully created.",
                        "Show to user the main information about the service.",
                    ],
                )

            case "list_available_service_types":
                response_services = await devopness.static.get_static_service_options()

                return MCPResponse.ok(
                    instructions=[
                        "Here are the available service types and versions:",
                        response_services.data.types,
                    ],
                )

            case "list_available_service_type_versions":
                if not service_type:
                    return MCPResponse.error(
                        [
                            "A valid service type is required to list available"
                            " service versions. Please ask the user to provide a valid"
                            " service type."
                        ]
                    )

                response_services = await devopness.static.get_static_service_options()

                service_type_versions = [
                    static_service_type.supported_versions
                    for static_service_type in response_services.data.types
                    if static_service_type.value == service_type
                ]

                return MCPResponse.ok(
                    instructions=[
                        "Here are the available service"
                        " versions for the selected type:",
                        service_type_versions,
                    ],
                )

    @staticmethod
    async def tool_deploy_service(
        operation: Literal[
            "deploy",
            "list_deploy_pipelines",
            "list_linked_servers",
        ],
        pipeline_id: int | None = None,
        service_id: int | None = None,
        server_ids: List[int] | None = None,
    ) -> MCPResponse[Action] | None:
        """
        Rules:
        1. DO NOT execute this tool without first confirming with the user which
          pipeline ID to use for deployment.

        2. DO NOT execute this tool without first confirming with the user which
          service ID to use for listing deployment pipelines or linked servers.

        Usage:

        1. To deploy a service:
            - Call this function with `operation='deploy'`, providing a valid
              `pipeline_id` and optionally `server_ids`.
            - Example: devopness_deploy_service(
                operation='deploy',
                pipeline_id=123,
                server_ids=[1, 2, 3]
            )

        2. To list deployment pipelines for a service:
            - Call this function with `operation='list_pipelines'` and provide a valid
              `service_id`.
            - Example: devopness_deploy_service(
                operation='list_pipelines',
                service_id=123
            )

        3. To list linked servers for a service:
            - Call this function with `operation='list_linked_servers'` and provide
              a valid `service_id`.
            - Example: devopness_deploy_service(
                operation='list_linked_servers',
                service_id=123
            )
        """
        await ensure_authenticated()

        match operation:
            case "deploy":
                if not pipeline_id:
                    return MCPResponse.error(
                        [
                            "A valid pipeline ID is required to deploy the service. "
                            "Please ask the user to provide a valid pipeline ID."
                        ]
                    )

                if server_ids and len(server_ids) == 0:
                    return MCPResponse.error(
                        [
                            "You provided an empty list of server IDs.",
                            "Provide a list with at least one server ID"
                            " to deploy the service.",
                            "Or not provide the server_ids argument to"
                            " deploy the service to all servers linked with the "
                            "service.",
                        ]
                    )

                action_pipeline_create: ActionPipelineCreatePlain = {}
                if server_ids:
                    action_pipeline_create["servers"] = server_ids

                deploy_response = await devopness.actions.add_pipeline_action(
                    pipeline_id,
                    action_pipeline_create,
                )

                return MCPResponse[Action].ok(
                    deploy_response.data,
                    [
                        "To monitor the deployment progress, visit the following URL:",
                        deploy_response.data.url_web_permalink,
                        "Explain to the user how to monitor the deployment progress.",
                        "Show the main information's about the action.",
                    ],
                )

            case "list_deploy_pipelines":
                if not service_id:
                    return MCPResponse.error(
                        [
                            "A service ID is required to list the deployment pipelines."
                            " Please ask the user to provide a service ID."
                        ]
                    )

                response = await devopness.pipelines.list_pipelines_by_resource_type(
                    service_id, "service"
                )

                if len(response.data) == 0:
                    return MCPResponse.error(
                        [
                            "No deployment pipelines were found for the given "
                            "service ID. "
                            "Please ask the user to verify the service and try again."
                        ]
                    )

                return MCPResponse.ok(
                    instructions=[
                        "The following deployment pipelines were found for this "
                        "service:",
                        response.data,
                        "Please ask the user to choose one of the listed pipeline IDs.",
                        "Then call this function again with the selected ID as the"
                        " 'pipeline_id' argument.",
                    ]
                )

            case "list_linked_servers":
                if not service_id:
                    return MCPResponse.error(
                        [
                            "A service ID is required to list the linked servers. "
                            "Please ask the user to provide a service ID."
                        ]
                    )

                list_linked_servers_response = (
                    await devopness.resource_links.list_resource_links_by_resource_type(
                        service_id,
                        "service",
                    )
                )

                if len(list_linked_servers_response.data) == 0:
                    return MCPResponse.error(
                        [
                            "No linked servers were found for the given service ID. "
                            "Please ask the user to verify the service and try again."
                        ]
                    )

                return MCPResponse.ok(
                    instructions=[
                        "The following servers are linked with this service:",
                        list_linked_servers_response.data,
                        "You can use these server IDs to deploy the service"
                        " to specific servers.",
                        "If you want to deploy the service to all linked servers, "
                        "you can provide an empty list of server IDs.",
                    ],
                )

    @staticmethod
    async def tool_list_services(
        environment_id: int,
    ) -> MCPResponse[List[ServiceRelation]]:
        """
        Rules:
        1. DO NOT execute this tool without first confirming with the user which
          environment ID to use.
        """
        await ensure_authenticated()
        response = await devopness.services.list_environment_services(environment_id)

        return MCPResponse.ok(
            response.data,
            [
                "Use the template below to format the list:",
                "{service.type_human_readable} (ID: {service.id})",
                "   - Version: {service.version}",
                "   - Last Action: {service.last_action.type_human_readable}"
                " ({service.last_action.status_human_readable})",
            ],
        )
